﻿module("VOICE")


local function roadnumber_explode(str)
	
	local t = {}
	
	wstring.gsub(str, L"([^/]+)/?", function(s) table.insert(t, s) end)
	
	for k,_ in ipairs(t) do
		
		if wstring.find(t[k], L"^%d+$") and t[k+1] and wstring.find(t[k+1], L"^[^%d]$") then
			t[k] = t[k] .. table.remove(t, k+1)
		end
		local v = t[k]
		
		v = wstring.gsub(v, L"^0%-?(%d+)$", function(route) local out = L"O" .. route t[k] = out return out	end)
		
		if wstring.find(v, L"-") then
			
			local fi = function (route1, route2) t[k] = route1 table.insert(t, k+1, route2) end
			
			wstring.gsub(v, L"^(%d+)-(%d+)$", fi)
			
			wstring.gsub(v, L"^(%a+%d+)-(%a+%d+)$", fi)
			
			wstring.gsub(v, L"^(%a+%d+)-(%d+)$", fi)
		end
	end
	return t
end

local patterns = {
	az = {L"^[aAáÁeEéÉiIíÍoOóÓöÖőŐuUúÚüÜűŰyY]", L"^[fFlLmMnNrRsSxXyY]%s?%d+", L"^1[-%s]", L"^1%d%d%d", L"^5%d*"},
	a = {L"^[bBcCdDfFgGhHjJkKlLmMnNpPqQrRsStTvVwWxXzZ][^%s%d]", L"^[bBcCdDgGhHjJkKpPqQtTvVwWzZ]%s?%d+", L"^1%d", L"^1%d%d", L"^1%d%d%d%d", L"^[2346789]%d*"},
	
	az_szam = {L"^1%s", L"^1%d%d%d", L"^5%d*"},
	az_szoveg = {L"^[aAáÁeEéÉiIíÍoOóÓöÖőŐuUúÚüÜűŰyY]", L"^[FLNRSXY]"},
	
	a_szam = {L"^1%d", L"^1%d%d", L"^1%d%d%d%d", L"^[2346789]%d*"},
	a_szoveg = {L"^[BCDGHJKPQTVWZ]"},
}

local function pattern_match(str, patterns)
	for _,pattern in ipairs(patterns) do
		if wstring.find(str, pattern) then return true end
	end
	return false
end

local function get_article(str)
	local article
	
	_,_, str = wstring.find(str, L"^%s*(.-)%s*$")
	if pattern_match(str, patterns.az) then article = L"az "
	elseif pattern_match(str, patterns.a) then article = L"a "
	else 
		voice_debug_log(L"Nincs nevelo match: " .. str, 2)
		article = L"a "
		ASSERT(false, L"Nincs nevelo match: \"" .. str .. L"\"") 
	end
	return article
end


local roadnumber_postfixes = {
	L"úton",
	L"autópályán",
	L"útra",
	L"autópályára",
	L"utat",
	L"autópályát",
	L"út felé",
	L"autópálya felé",
}

local function inflection_roadnumber(str, inflection_typ)
	
	local routes = roadnumber_explode(str)
	
	local highway = true
	
	for k,v in ipairs(routes) do
		highway = highway and wstring.find(v, L"^[AM]%d+$")
		
		v = wstring.gsub(v, L"-", L" ")
		
		v = wstring.gsub(v, L"(%a)(%d+)", L"%1 %2")
		
		v = wstring.gsub(v, L"(%d+)(%a)", L"%1 %2")
		
		v = v .. L"-s"
		
		routes[k] = v
	end
	local postfix = L""
	if inflection_typ > 0 then 
		local com = inflection_typ - 1 
		if highway then highway = 2
		else highway = 1 end
		postfix = L" " .. roadnumber_postfixes[2*com + highway]
	end
	local route = table_concat(routes, L", ") .. postfix
	return get_article(route)..route
end


local function signpost_exitnumber(data, idx)
	local exitnumber = data[idx].signpost.exitnumber
	if exitnumber then 
		local str = exitnumber.text
		
		str = get_article(str) .. str .. L"-s kijáraton"
		return str
	end
end

local function signpost_exitname(data, idx)
	local exitname = data[idx].signpost.exitname
	if exitname and not exitname.phoneme then 
		local str = exitname.text
		
		if pattern_match(str, patterns.az_szoveg) then 
			if wstring.find(str, L"[iI]$") then str = L"az " .. str 
			else str = L"az " .. str .. L"-i" end
		elseif pattern_match(str, patterns.az_szam) then str = L"az " .. str .. L"-s"
		elseif pattern_match(str, patterns.a_szoveg) then 
			if wstring.find(str, L"[iI]$") then str = L"a " .. str 
			else str = L"a " .. str .. L"-i" end
		elseif pattern_match(str, patterns.a_szam) then str = L"a " .. str .. L"-s"
		else ASSERT(false) end
		str = str .. L" kijáraton"
		return str
	end
end

local function signpost_destination(data, idx)
	local dest = data[idx].signpost.destination
	if dest then
		local str
		local article = get_article(dest.text)
		if not dest.phoneme then 
			str = transform.country_abbrev_table:transform(dest.text)
			if str ~= dest.text then
				article = L""
			end
		else 
			str = dest.phoneme 
		end
		return  article .. str .. L" felé"
	end
end

local function signpost_settlement(data, idx)
	local settlement = data[idx].signpost.settlement
	if settlement then
		local str
		if not settlement.phoneme then 
			str = transform.settlement_exceptions_table:transform(settlement.text)
		else 
			str = settlement.phoneme 
		end
		return str .. L" felé"
	end
end

local function signpost_roadnumber(data, idx)
	
	local roadnumber = data[idx].signpost.roadnumber
	if roadnumber then
		local inf_type 
		
		if data[idx].signpost.sign_road_branch then inf_type = 2
		else inf_type = 4 end
		return inflection_roadnumber(roadnumber.text, inf_type)
	end
end

local function road_name(data, idx)
	local name = data[idx].road.name
	if name then 
		local str
		
		str = transform.road_abbrev_table:transform(name.text)
		
		local article = get_article(name.text)
		if not name.phoneme then 
			
			local nr 
			local com = com_groups[data[idx].manouver]
			if com == 1 then
				str,nr = transform.com1:transform(str)
			elseif com == 2 then
				str,nr = transform.com2:transform(str)
			elseif com == 3 then
				str,nr = transform.com3:transform(str)
			end
			
			if nr == 0 then str = str .. L" felé" end
                        
                        article = get_article(str)
		else
			str = name.phoneme .. L" felé"		
		end
		return article..str
	end
end

local function road_number(data, idx)
	local number = data[idx].road.number
	if number then 
		return inflection_roadnumber(number.text, com_groups[data[idx].manouver])
	end
end


local function chain(t, trafo, data, idx)
	local out = trafo(data, idx)
	if out then table.insert(t, out) end
	return t
end

function format_destname(data, idx)
	local t
	if data[idx].signpost then
		t = chain({}, signpost_exitname, 	data, idx)
		t = chain(t,  signpost_exitnumber, 	data, idx)
		t = chain(t,  signpost_roadnumber, 	data, idx)
		t = chain(t,  signpost_destination,	data, idx)
		t = chain(t,  signpost_settlement, 	data, idx)
	elseif data[idx].road then
		t = chain({}, road_number, 	data, idx)
		t = chain(t,  road_name, 	data, idx)
	end
	return table_concat(t, L", ")
end

function format_streetname(data, idx)
	local t
	t = chain({}, road_number, 	data, idx)
	t = chain(t,  road_name, 	data, idx)
	return table_concat(t, L", ")
end

function format_sentence(str)
	return str
end


function route_summary_format_road_name(data)
	local routes = roadnumber_explode(data.text)
	
	table.sort(routes)
	
	for k,v in ipairs(routes) do
		
		v = wstring.gsub(v, L"-", L" ")
		
		v = wstring.gsub(v, L"(%a)(%d+)", L"%1 %2")
		
		v = wstring.gsub(v, L"(%d+)(%a)", L"%1 %2")
		
		v = v .. L"-s"
		
		routes[k] = v
	end
	return table_concat(routes, L", ") .. L" út"
end

function route_summary_format_street_name(data)
	
	return transform.route_summary_abbrev:transform(data.text) 
end

function route_summary_format_bridge_tunnel(data)
	return transform.road_abbrev_table:transform(data.text) 
end

function route_summary_format_order(data)
	return transform.road_abbrev_table:transform(data.text) 
end


function traffic_event_supported()
	return true
end

local tf_table = {
	{false, L"[lL]ánchíd", L"lánc híd"},
	
	{true, L"^(%a)(%d+)", L"%1 %2-s"},
	
	{true, L"^(%d+)%.?", L"%1-s"},
	
	{true, L"%((%d+)%.?", L"(%1-s"},
}

local function tf(str, article)
	local out = str
	local road = 0
	local nr
	for _,v in ipairs(tf_table) do 
		out, nr = wstring.gsub(out, v[2], v[3]) 
		
		if v[1] then road = road + nr end
	end
	
	if article or road > 0 then return get_article(str)..out
	else return out end
end

function traffic_event(DescKey, data)
	local str = translate_voice(DescKey)
	ASSERT(MODEL.regional.is_it_voice_localizable(DescKey), "Missing TrafficEvent dictionary.voice key:" .. DescKey)
	
	if data.roadnumber then 
		local road = inflection_roadnumber(data.roadnumber.text, 1)
		if data.from and not data.to then
		
			str =  str..L", "..road..L", "..tf(data.from.text)..L" közelében."
		elseif data.from and data.to then 
		
			str =  str..L", "..road..L", "..tf(data.from.text)..L" és "..tf(data.to.text)..L" között."
		else
		
			str = str..road..L"."
		end
	
	elseif data.roadname then
		local road = data.roadname.text
		
		road = get_article(road) .. transform.com1:transform(road) 
		if data.from and not data.to then
			
			local _,_, from, to = wstring.find(data.from.text, L"^([^,]+),(.+)$")
			if from and to then 
				str = str..L", "..road..L", "..tf(from, true)..L" és "..tf(to,true)..L" kereszteződésében."
			else 
				str = str..L", "..road..L", "..tf(data.from.text,true)..L" kereszteződésben."
			end
		elseif data.from and data.to then
			str = str..L", "..road..L", "..tf(data.from.text,true)..L" és "..tf(data.to.text,true)..L" között."
		else
			str = str..road..L"."
		end
	
	else
		if data.from and not data.to then
		
			str =  str..L", "..tf(data.from.text)..L" közelében."
		elseif data.from and data.to then 
		
			str =  str..L", "..tf(data.from.text)..L" és "..tf(data.to.text)..L" között."
		else
		
			str = str..L"."
		end
	end
	return str
end
